/*
 CoronaLights from rectangle splines.
 Please note this script will work only with a single spline object.
 
 2017-04-24: Initial code by Clment "Sugz" Plantec (plantec.clement@gmail.com), made for VRayLights
 2018-04-23: Reworked by Nicolas Caplat (nicocaplat@gmail.com), to work with Corona Lights

USE AT YOUR OWN RISK !
*/

macroScript CLightsFromSplines Category:"Corona Renderer" toolTip:"Creates CoronaLights from rectangle splines" buttontext:"CLightsSplines"
(
	local theSpline
	local rectangles = #()
	local theLights = #()
	local LightsMasterCA
	local myDummy
	
	-- FUNCTIONS--
	fn SetLightMasterCA = 
		(
			LightsMasterCA =  attributes LightsMaster
			(
				-- Get the node this custom attribute is attached to ------------------------------------------------------------------------------------------------------------------------------------------------------
				fn GetNode = (refs.dependentnodes (custattributes.getowner this) firstonly:on)

				parameters params rollout:view
				(
					intensity type:#float default:2000 ui:intensitySpn
					temp type:#float default:6500 ui:tempSpn
					direct type:#float default:0.0 ui:directSpn
				)

				rollout view "Lights Master"
				(
					spinner intensitySpn "Intensity" type:#float range:[0,9999999999,2000]
					spinner tempSpn "Temperature" type:#float range:[0,30000,6500]
					spinner directSpn "Directionality" type:#float range:[0,1,0]

					fn SetLightsProps prop arg = 
					(
						obj = GetNode()
						for c in obj.children do 
						(
							case prop of 
							(
								#intensity: c.intensity = arg
								#temperature: c.blackbodyTemp = arg
								#directionality: c.directionality = arg
							)
						)
					)

					on intensitySpn changed val do SetLightsProps #intensity val
					on tempSpn changed val do SetLightsProps #temperature val
					on directSpn changed val do SetLightsProps #directionality val

				) -- End rollout
			)
		)
		
		
	-- SELECTION --	
	theSpline = $
	if classOf theSpline != SplineShape then
	(
		return messageBox "Please select a spline object !" title:"!! Warning !!"
	)

	centerPivot theSpline 
	
	-- CONVERSION of ELEMENTS --	
	ss = convertToPoly (snapshot theSpline)
	ss.name = "temp"
	while polyOp.getNumFaces ss != 0 do 
	(
		faceList = polyOp.getElementsUsingFace ss #{1}
		theName = uniqueName theSpline.name
		canDetach = polyOp.detachFaces ss faceList asNode:true name:theName
		if canDetach then 
		(
			newObj = getNodeByName theName
			centerPivot newObj
			worldAlignPivot newObj
			resetXForm newObj
			convertToPoly newObj
			append rectangles newObj
		)
	)
	delete ss
	
	
	-- LIGHTS CREATION --
	for rect in rectangles do 
	(
		vertPos1 = polyOp.getVert rect 1
		vertPos2 = polyOp.getVert rect 2
		vertPos4 = polyOp.getVert rect 4
		
		_light = CoronaLight colorMode:1 shape:1 blackbodyTemp:6500 intensityUnits:1 intensity:2000 viewportWireframe:on visibleDirectly:off visibleRefl:off visibleRefr:off gizmoDisplaySize: 5
		_light.width = (distance vertPos2 vertPos1)
		_light.height= (distance vertPos4 vertPos1)
		_light.pos = rect.pos
		_light.dir = vertPos2 - vertPos1
		append theLights _light
	)
	
	for _light in theLights do 
	(
		currentMatrix = _light.transform
		preRotate currentMatrix (eulertoquat (eulerAngles 90 0 0))
		preRotate currentMatrix (eulertoquat (eulerAngles 0 0 90))
		preRotate currentMatrix (eulertoquat (eulerAngles 0 180 0))
		_light.transform = currentMatrix				
	)
	
	-- DUMMY CONTROL--
	myDummy = point pos:theSpline.pos box:true size:200 drawOnTop: true name:"LightsCONTROL"
	for _light in theLights do _light.parent = myDummy

	SetLightMasterCA()
	custAttributes.add myDummy LightsMasterCA
	
	delete rectangles
	select myDummy
)
